/* basil.c */
/*-
 * Copyright (c) 2004 AIAN
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id:$
 */

#include <stdio.h>
#include <unistd.h>

#include "serial.h"
#include "basil.h"

int basil_open(const char * com_open);
void basil_close(int fd);
int basil_reset(int fd);
int address_set(int fd, short int address);
unsigned char read_rom(int fd, short int address);
unsigned char read_rom_with_address_increment(int fd);
int write_rom(int fd, short int address, unsigned char data);

int
basil_open(const char * port)
{
    int fd;

    fd = com_open(port);
    com_set_attribute(fd);

    return fd;
}

void
basil_close(int fd)
{
    basil_reset(fd);
    com_close(fd);
}

int
basil_reset(int fd)
{
    com_putc(fd, 'u');

    return 0;
}

int
address_set(int fd, short int address)
{
    unsigned char address_high;
    unsigned char address_low;

    address_high = (unsigned char)((address & 0xFF00) >> 8);
    address_low = (unsigned char)(address & 0x00FF);

    com_putc(fd, 'a');
    com_putc(fd, address_high);
    com_putc(fd, address_low);

    return 0;
}

unsigned char
read_rom(int fd, short int address)
{
    unsigned char data;

    address_set(fd, address);
    com_putc(fd, 'r');
    data = com_getc(fd);

    return data;
}

int
read_rom_block(int fd, unsigned char * block)
{
    com_putc(fd, 'b');
    com_get_bytes(fd, block, BLOCK_BYTES);

    return 0;
}

unsigned char
read_rom_with_address_increment(int fd)
{
    unsigned char data;

    com_putc(fd, 'p');
    data = com_getc(fd);

    return data;
}

int
write_rom(int fd, short int address, unsigned char data)
{
    address_set(fd, address);
    com_putc(fd, 'w');
    com_putc(fd, data);

    return 0;
}
