/*-
 * Copyright (c) 2004 AIAN
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id:$
 */

#include <sys/ioctl.h>
#include <sys/types.h>

#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h>
#include <unistd.h>

#include "serial.h"

int
com_open(const char * port)
{
    int fd;

    /* REFERENCE: termios(4)
     *
     *   Under normal circumstances, a call to the open() function
     *   waits for the modem connection to complete.  However, if the
     *   O_NONBLOCK flag is set or if CLOCAL has been set, the open()
     *   function returns immediately without waiting for the
     *   connection.
     */
    if ((fd = open(port, O_RDWR | O_NOCTTY | O_NONBLOCK)) < 0) {
        perror(port);
        exit(1);
    }
    return fd;
}

int
com_close(int fd)
{
    close(fd);
    return 0;
}

int
com_set_attribute(int fd)
{
    struct termios tio;
    int flags;

    tcgetattr(fd, &tio);

    cfsetispeed(&tio, B115200);
    cfsetospeed(&tio, B115200);
    
    /* REFERENCE: termios(4)
     *
     *   The CCTS_OFLOW (CRTSCTS) flag is currently unused.
     */
    tio.c_cflag &= ~(CSIZE | CSTOPB | PARENB | PARODD | HUPCL | CRTSCTS | CCTS_OFLOW | CRTS_IFLOW);
    tio.c_cflag |= (CS8 | CREAD | CLOCAL);
    tio.c_iflag = 0;
    tio.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON | IXOFF);
    /* tio.c_iflag |= ~(IXON); */
    tio.c_oflag = 0;
    tio.c_lflag = 0;
    tio.c_cc[VTIME] = 0;
    tio.c_cc[VMIN] = 1;

    tcflush(fd, TCIOFLUSH);
    tcsetattr(fd, TCSANOW, &tio);

    flags = fcntl(fd, F_GETFL, 0);
    fcntl(fd, F_SETFL, flags & ~O_NONBLOCK);

    return 0;
}

int
com_putc(int fd, char buffer)
{
    int flag;

    flag = write(fd, &buffer, 1);

    if (flag == -1) {
	fprintf(stderr, "COM sending data error\n");
	exit(-1);
    }
    if (flag != 1) {
	fprintf(stderr, "COM sending data incorect\n");
	exit(-1);
    }
    return flag;
}

unsigned char
com_getc(int fd)
{
    unsigned char buf;

    while ((read(fd, &buf, 1)) <= 0);

    return buf;
}

int
com_get_bytes(int fd, unsigned char * buffer, int nbyte)
{
    int  flag, i;
    unsigned char ch = 0;
    
    nbyte = 0x100;
    i = 0;
    do{
        flag = read(fd, &ch, 1);
        if (flag == -1){
            fprintf(stderr,"flag error\n");
            return -1;
        }
        if(flag > 0){
            *(buffer + i) = ch;
            i++;
        }
    }while(i < nbyte);

    return 0;
}
